package com.gic.auth.service;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.dto.UserListDTO;
import com.gic.auth.qo.UserListQO;

import java.util.List;

/**
 * @author guojx
 * @date 2019/7/16 6:48 PM
 */
public interface UserApiService {

    /**
     * 新增用户
     * @param userDTO
     * @return
     */
    ServiceResponse<Integer> saveUser(UserDTO userDTO);

    /**
     * 新增运营实施管理员
     * @Title: saveOperationUser 
     * @Description:
      * @author guojuxing
     * @param userDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
      */
    ServiceResponse<Integer> saveOperationUser(UserDTO userDTO);

    /**
     * 更新运营人员信息,，主要是名称修改
     * @Title: updateOperationUser 
     * @Description:
     * @author guojuxing
     * @param userId
     * @param userName
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    ServiceResponse<Void> updateOperationUser(Integer userId, String userName);

    /**
     * 修改职位（个人资料）
     * @Title: updateUserPosition 
     * @Description:
     * @author guojuxing
     * @param userId
     * @param userPosition 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    ServiceResponse<Void> updateUserPosition(Integer userId, Integer userPosition);

    /**
     * 新增超级用户
     * @Title: saveAdmin 
     * @Description:
      * @author guojuxing
     * @param userDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
      */
    ServiceResponse<Integer> saveAdmin(UserDTO userDTO);

    /**
     * 编辑
     * @param userDTO
     * @return
     */
    ServiceResponse editUser(UserDTO userDTO);

    /**
     * 编辑超级管理员
     * @param userDTO
     * @return
     */
    ServiceResponse<Void> editAdmin(UserDTO userDTO);

    /**
     * 用户详情（包括资源组、权限等待信息）
     * @Title: getUserById 
     * @Description:
      * @author guojuxing
     * @param userId 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.auth.dto.UserDTO> 
      */
    ServiceResponse<UserDTO> getUserById(Integer userId);

    /**
     * 用户基本信息
     * @Title: getUserInfoById 
     * @Description:
      * @author guojuxing
     * @param userId 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.auth.dto.UserDTO> 
      */
    ServiceResponse<UserDTO> getUserInfoById(Integer userId);

    /**
     * 删除
     * @Title: delete 
     * @Description:
      * @author guojuxing
     * @param userId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void>  返回日志操作对象
      */
    ServiceResponse<String> delete(Integer userId);

    /**
     * 企业查询超级管理员
     * @param enterpriseId
     * @return
     */
    ServiceResponse<UserDTO> getUserByEnterpriseId(Integer enterpriseId);

    /**
     * 根据登录账号查询商户列
     * @Title: listUserByPhoneNumber 
     * @Description:
     * @author guojuxing
     * @param phoneNumber
     * @param nationCode 国际区码
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.UserDTO>> 
     */
    ServiceResponse<List<UserDTO>> listUserByPhoneNumber(String phoneNumber, String nationCode);

    /**
     * 手机号和企业查询用户，给好办用的
     * @Title: listUserByPhoneNumber 
     * @Description:
      * @author guojuxing
     * @param phoneNumber
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.UserDTO>> 
      */
    ServiceResponse<UserDTO> listUserByPhoneNumber(String phoneNumber, Integer enterpriseId);

    /**
     * listUserByPhoneNumber 
     * @Title: listUserByPhoneNumber
     * @Description: 
     * @author zhiwj 
     * @param nationCode
     * @param phoneNumber
     * @param enterpriseId
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.auth.dto.UserDTO>
     * @throws 
     */
    ServiceResponse<UserDTO> listUserByPhoneNumber(String nationCode, String phoneNumber, Integer enterpriseId);

    /**
     * 获取运维账号
     * @Title: getUserByPhoneNumber 
     * @Description:
     * @author guojuxing
     * @param nationCode 手机区号
     * @param phoneNumber 手机号码
     * @param enterpriseId 商户ID
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.auth.dto.UserDTO> 
     */
    ServiceResponse<UserDTO> getUserByPhoneNumber(String nationCode, String phoneNumber, Integer enterpriseId);
    
    /**
     * login 
     * @Title: login
     * @Description: 
     * @author zhiwj 
     * @param nationCode
     * @param phoneNumber
     * @param enterpriseId
     * @param password
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.auth.dto.UserDTO>
     * @throws 
     */
    ServiceResponse<UserDTO> login(String nationCode, String phoneNumber, Integer enterpriseId, String password);

    /**
     * 分页查询管理员列表数据
     * @Title: pageUser 
     * @Description:
      * @author guojuxing
     * @param params 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page<com.gic.auth.dto.UserDTO>> 
      */
    ServiceResponse<Page<UserListDTO>> pageUser(UserListQO params);

    /**
     * 不分页查询列表
     * @Title: listUser
     * @Description:
     * @author zhiwj
     * @param params
     * @return com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.UserListDTO>>
     * @throws
     */
    ServiceResponse<List<UserListDTO>> listUser(UserListQO params);

    /**
     * 根据用户ID list查询用户信息
     * @Title: listUserByIdList 
     * @Description:
      * @author guojuxing
     * @param userIdList 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.UserDTO>> 
      */
    ServiceResponse<List<UserDTO>> listUserByIdList(List<Integer> userIdList);
    
    /**
     * getUserDetail 
     * @Title: getUserDetail
     * @Description: 
     * @author zhiwj 
 * @param token
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Object>
     * @throws 
     */
    ServiceResponse<Object> getUserDetail(String token);

    /**
     * 移除对应的分组
     * @Title: removeAccountGroup 
     * @Description:
      * @author guojuxing
     * @param accountGroupId
     * @param userId 
     * @param type
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.String>  返回日志操作对象
      */
    ServiceResponse<String> removeAccountGroup(Integer accountGroupId, Integer userId, Integer type);

    /**
     * 编辑修改个人资料：手机号码/密码
     * @Title: editPhoneOrPassword 
     * @Description:
      * @author guojuxing
     * @param userId
     * @param nationCode
     * @param phone
     * @param password 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.String> 
      */
    ServiceResponse<String> editPhoneOrPassword(Integer userId, String nationCode, String phone, String password);

    /**
     * 查询所有的用户，gic管理员/运营实施管理员
     * @Title: listAllUser 
     * @Description:
      * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<java.lang.Integer>> 
      */
    ServiceResponse<List<Integer>> listAllUser(Integer enterpriseId);

    /**
     * 查询gic用户列表
     * @Title: listGicUser 
     * @Description:
      * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.UserDTO>> 
     */
    ServiceResponse<List<UserDTO>> listGicUser(Integer enterpriseId);

    /**
     * 运维账户
     * @Title: listOperationUser 
     * @Description:
     * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.UserDTO>> 
     */
    ServiceResponse<List<UserDTO>> listOperationUser(Integer enterpriseId);
    /**
     * 查询全部
     * @Title: listUser 
     * @Description:
     * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.UserDTO>> 
     */
    ServiceResponse<List<UserDTO>> listUser(Integer enterpriseId);
}
