package com.gic.auth.dao.mapper;

import com.gic.auth.dto.AuthItemListDTO;
import com.gic.auth.dto.MenuDTO;
import com.gic.auth.entity.TabSysMenu;
import com.gic.auth.qo.MenuListQO;
import com.gic.auth.qo.MenuOperationItemListQO;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 
 * @ClassName: TabSysMenuMapper
 * @Description:  
 * @author zhiwj
 * @date 2020-09-10 9:41
 */
public interface TabSysMenuMapper {


    /**
     * 插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insert(TabSysMenu record);

    /**
     * 动态插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insertSelective(TabSysMenu record);

    /**
     * 根据主键查询
     *
     * @param menuId 主键
     * @return 实体对象
     */
    TabSysMenu selectByPrimaryKey(Integer menuId);

    /**
     * 根据主键动态更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKeySelective(TabSysMenu record);

    /**
     * 更新好办小程序应用
     * @Title: updatePageForHaobanApplet 
     * @Description:
     * @author guojuxing
     * @param record 
     * @return  int 
     */
    int updatePageForHaobanApplet(TabSysMenu record);

    /**
     * 根据主键更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKey(TabSysMenu record);

    /**
     * 查询菜单树（页面、子页面、权限项）
     * @Title: listMenu 
     * @Description:
      * @author guojuxing
     * @param params 
     * @return  java.util.List<com.gic.auth.entity.TabSysMenu> 
      */
    List<TabSysMenu> listMenu(MenuListQO params);

    /**
     * 获取最大排序值
     * @Title: getMaxSortByParentId 
     * @Description:
      * @author guojuxing
     * @param parentId  父级ID
     * @param project
     * @return  int 
      */
    int getMaxSortByParentId(@Param("parentId") Integer parentId, @Param("project") String project);

    /**
     * 查询页面下的操作项列表数据
     * @Title: listMenuItemByParentId 
     * @Description:
      * @author guojuxing
     * @param parentId 
     * @return  java.util.List<com.gic.auth.entity.TabSysMenu> 
      */
    List<TabSysMenu> listMenuItemByParentId(@Param("parentId") Integer parentId);

    /**
     * 查询页面下的菜单数据
     * @Title: listMenuByParentId 
     * @Description:
      * @author guojuxing
     * @param parentId 
     * @param project
     * @return  java.util.List<com.gic.auth.entity.TabSysMenu> 
      */
    List<TabSysMenu> listMenuByParentId(@Param("parentId") Integer parentId, @Param("project") String project);

    /**
     * 分页查询操作项
     * @Title: pageMenuOperationItem 
     * @Description:
      * @author guojuxing
     * @param params 
     * @return  java.util.List<com.gic.auth.entity.TabSysMenu> 
      */
    List<TabSysMenu> pageMenuOperationItem(MenuOperationItemListQO params);

    /**
     * 查询
     * @Title: selectByIds 
     * @Description:
      * @author guojuxing
     * @param menuIdList 
     * @return  java.util.List<com.gic.auth.entity.TabSysMenu> 
      */
    List<TabSysMenu> selectByIds(@Param("menuIdList") List<Integer> menuIdList);

    /**
     * listByParentMenuIdList
     * @Title: listByParentMenuIdList
     * @Description:
     * @author zhiwj
     * @param parentMenuIdList
     * @return java.util.List<com.gic.auth.entity.TabSysMenu>
     * @throws
     */
    List<TabSysMenu> listByParentMenuIdList(@Param("parentMenuIdList") List<Integer> parentMenuIdList);

    /**
     * gic菜单转应用菜单的时候，批量进行修改相关数据
     * @Title: updateMenuTmp 
     * @Description:
     * @author guojuxing
     * @param menuIdList
     * @param project
     * @param parentId
     * @param parentCode
     * @param menuVersion
     * @param menuVersionName 
     * @return  void 
     */
    void updateMenuTmp(@Param("menuIdList") List<Integer> menuIdList,
                       @Param("project") String project,
                       @Param("parentId") Integer parentId,
                       @Param("parentCode") String parentCode,
                       @Param("menuVersion") String menuVersion,
                       @Param("menuVersionName") String menuVersionName);

    /**
     * selectByIdsAndOrderBy
     * @Title: selectByIdsAndOrderBy
     * @Description:
     * @author zhiwj
     * @param menuIdList
     * @param orderByStr
     * @return java.util.List<com.gic.auth.entity.TabSysMenu>
     * @throws
     */
    List<TabSysMenu> selectByIdsAndOrderBy(@Param("menuIdList")List<Integer> menuIdList, @Param("orderByStr") String orderByStr);

    /**
     * 查询页面下的权限项数据
     * @Title: listAuthItemByMenuId 
     * @Description:
      * @author guojuxing
     * @param menuId 
     * @return  java.util.List<com.gic.auth.dto.AuthItemListDTO> 
      */
    List<AuthItemListDTO> listAuthItemByMenuId(@Param("menuId") Integer menuId);

    /**
     * 统计页面下的子项目
     * @Title: countChildByParentId 
     * @Description:
      * @author guojuxing
     * @param menuId 
     * @return  int 
      */
    int countChildByParentId(@Param("menuId") Integer menuId);

    /**
     * 查询菜单树（页面&权限项）
     * @Title: listMenuTree 
     * @Description:
      * @author guojuxing
     * @param project 
     * @param authTypeList
     * @param versionCode gic商户基础服务版本
     * @return  java.util.List<com.gic.auth.entity.TabSysMenu> 
      */
    List<TabSysMenu> listMenuTree(@Param("project") String project,
                                  @Param("versionCode") String versionCode,
                                  @Param("list") List<Integer> authTypeList);

    /**
     * selectByProjectList
     * @Title: selectByProjectList
     * @Description:
     * @author zhiwj
     * @param projectList
     * @return java.util.List<com.gic.auth.entity.TabSysMenu>
     * @throws
     */
    List<TabSysMenu> selectByProjectList(@Param("projectList") List<String> projectList);

    /**
     * 查询应用权限
     * @Title: selectAppMenuByParam 
     * @Description: 
      * @author guojuxing 
     * @param projectList appID List
     * @param versionList  版本/拓展包 list
     * @param authTypeList
     * @return  java.util.List<com.gic.auth.entity.TabSysMenu> 
      */
    List<TabSysMenu> selectAppMenuByParam(@Param("projectList") List<String> projectList,
            @Param("versionList") List<String> versionList, @Param("authTypeList") List<Integer> authTypeList);

    /**
     * 相同code个数
     * @Title: countByMenuCode 
     * @Description:
      * @author guojuxing
     * @param menuCode 
     * @param menuId
     * @return  int 
      */
    int countByMenuCode(@Param("menuCode") String menuCode, @Param("menuId") Integer menuId);

    /**
     * getMenuByMenuCode
     * @Title: getMenuByMenuCode
     * @Description:
     * @author zhiwj
     * @param menuCode
     * @return com.gic.auth.entity.TabSysMenu
     * @throws
     */
    TabSysMenu getMenuByMenuCode(String menuCode);

    /**
     * 批量更新子集的父级code数据，根据父级ID
     * @Title: updateParentCodeByParentId 
     * @Description:
      * @author guojuxing
     * @param parentId 
     * @param parentCode
     * @return  void 
      */
    void updateParentCodeByParentId(@Param("parentId") Integer parentId, @Param("parentCode") String parentCode);

    /**
     * 仅超管可用的操作项
     * @Title: listAdminOnlyOperationItem 
     * @Description:
      * @author guojuxing  
     * @return  java.util.List<com.gic.auth.entity.TabSysUser> 
      */
    List<TabSysMenu> listAdminOnlyOperationItem();

    /**
     * deleteByMenuIdList 
     * @Title: deleteByMenuIdList
     * @Description: 
     * @author zhiwj 
     * @param menuIdList
     * @return void
     * @throws 
     */
    void deleteByMenuIdList(@Param("list") List<Integer> menuIdList);

    /**
     * 逻辑删除不存在的应用菜单
     * @Title: deleteNotInApp 
     * @Description:
     * @author guojuxing
     * @param projectList 
     * @param platformTypeList 
     * @return  void 
     */
    void deleteNotInApp(@Param("list") List<String> projectList, @Param("platformTypeList") List<Integer> platformTypeList);


    /**
     * listMenuTemp
     * @Title: listMenuTemp
     * @Description:
     * @author zhiwj
     * @param project
     * @return java.util.List<com.gic.auth.entity.TabSysMenu>
     * @throws
     */
    List<TabSysMenu> listMenuTemp(String project);

    /**
     * listMenuTempByMenuCode
     * @Title: listMenuTempByMenuCode
     * @Description:
     * @author zhiwj
     * @param menuCode
     * @return java.util.List<com.gic.auth.entity.TabSysMenu>
     * @throws
     */
    List<TabSysMenu> listMenuTempByMenuCode(String menuCode);

    /**
     * listMenuTempByMenuIdList
     * @Title: listMenuTempByMenuIdList
     * @Description:
     * @author zhiwj
     * @param menuIdList
     * @return java.util.List<com.gic.auth.entity.TabSysMenu>
     * @throws
     */
    List<TabSysMenu> listMenuTempByMenuIdList(@Param("list") List<Integer> menuIdList);

    /**
     * getAuthItemByDevMenuId
     * @Title: getAuthItemByDevMenuId
     * @Description:
     * @author zhiwj
     * @param menuId
     * @return java.util.List<com.gic.auth.entity.TabSysMenu>
     * @throws
     */
    List<TabSysMenu> getAuthItemByDevMenuId(Integer menuId);
}