package com.gic.auth.service.impl;

import com.gic.auth.constant.DeleteFlagConstants;
import com.gic.auth.constant.MenuPlatformTypeEnum;
import com.gic.auth.dao.mapper.TabSysMenuMapper;
import com.gic.auth.dto.AuthItemListDTO;
import com.gic.auth.dto.MenuDTO;
import com.gic.auth.entity.TabSysMenu;
import com.gic.auth.qo.MenuListQO;
import com.gic.auth.qo.MenuOperationItemListQO;
import com.gic.auth.service.MenuService;
import com.gic.commons.util.EntityUtil;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 
 * @ClassName: MenuServiceImpl
 * @Description:  
 * @author zhiwj
 * @date 2020-09-10 9:38
 */
@Service("menuService")
public class MenuServiceImpl implements MenuService {
    @Autowired
    private TabSysMenuMapper tabSysMenuMapper;
    @Override
    public List<TabSysMenu> listMenu(MenuListQO params) {
        if (params.getPlatformType() == null) {
            params.setPlatformType(0);
        }
        return this.tabSysMenuMapper.listMenu(params);
    }

    @Override
    public TabSysMenu getMenuById(Integer menuId) {
        return this.tabSysMenuMapper.selectByPrimaryKey(menuId);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public int savePage(MenuDTO menuDTO) {
        TabSysMenu record = EntityUtil.changeEntityNew(TabSysMenu.class, menuDTO);
        record.setCreateTime(new Date());
        record.setUpdateTime(new Date());
        if (record.getPlatformType() == null) {
            //默认gic平台菜单
            record.setPlatformType(0);
        }
        record.setDeleteFlag(DeleteFlagConstants.NORMAL_STATUS);
        if (StringUtils.isBlank(menuDTO.getMenuCode())) {
            //权限项没有菜单code数据，但是这样会导致不好批量上传到生产，给一个默认的数据
            record.setMenuCode(menuDTO.getParentCode() == null ? "noParentCode" : menuDTO.getParentCode() + "-" + menuDTO.getLevel() + menuDTO.getSort());
        }
        tabSysMenuMapper.insertSelective(record);
        return record.getMenuId();
    }

    @Override
    public int getMaxSortByParentId(Integer parentId,  String project) {
        return tabSysMenuMapper.getMaxSortByParentId(parentId, project);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updatePage(MenuDTO menuDTO) {
        TabSysMenu record = EntityUtil.changeEntityNew(TabSysMenu.class, menuDTO);
        record.setUpdateTime(new Date());
        tabSysMenuMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public void updatePageForHaobanApplet(MenuDTO menuDTO) {
        TabSysMenu record = EntityUtil.changeEntityNew(TabSysMenu.class, menuDTO);
        record.setUpdateTime(new Date());
        record.setPlatformType(MenuPlatformTypeEnum.HAOBAN_APPLET_APP.getCode());
        tabSysMenuMapper.updatePageForHaobanApplet(record);
    }

    @Override
    public List<TabSysMenu> listMenuItemByParentId(Integer parentId) {
        List<TabSysMenu> list = tabSysMenuMapper.listMenuItemByParentId(parentId);
        if (CollectionUtils.isNotEmpty(list)) {
            return list;
        }
        return new ArrayList<>();
    }

    @Override
    public void setSort(TabSysMenu record, Integer sortValue) {
        List<TabSysMenu> list = tabSysMenuMapper.listMenuByParentId(record.getParentId(), record.getProject());
        Integer fromSortValue = record.getSort();
        if (fromSortValue > sortValue) {
            //向上拖拽
            for (int i = 0, length = list.size(); i < length; i++) {
                TabSysMenu temp = list.get(i);
                //如果大于sortValue，都需要降低排序值,往后推
                boolean isNeedDown = temp.getSort() >= sortValue && temp.getSort() < fromSortValue;
                if (isNeedDown) {
                    updateSort(list.get(i + 1).getSort(), temp.getMenuId());
                }
            }
        } else if (fromSortValue < sortValue) {
            //向下拖拽
            for (int i = 0, length = list.size(); i < length; i++) {
                TabSysMenu temp = list.get(i);
                boolean isNeedUp = temp.getSort() <= sortValue && temp.getSort() > fromSortValue;
                if (isNeedUp) {
                    updateSort(list.get(i - 1).getSort(), temp.getMenuId());
                }
            }
        }
        updateSort(sortValue, record.getMenuId());
    }

    @Override
    public Page<TabSysMenu> pageMenuOperationItem(MenuOperationItemListQO params) {
        PageHelper.startPage(params.getCurrentPage(), params.getPageSize());
        if (params.getPlatformType() == null) {
            params.setPlatformType(0);
        }
        List<TabSysMenu> list = tabSysMenuMapper.pageMenuOperationItem(params);
        return (Page<TabSysMenu>) list;
    }

    @Override
    public List<TabSysMenu> selectByIds(List<Integer> menuIdList) {
        if (CollectionUtils.isEmpty(menuIdList)) {
            return new ArrayList<>();
        }
        return tabSysMenuMapper.selectByIds(menuIdList);
    }

    @Override
    public List<TabSysMenu> selectByIdsAndOrderBy(List<Integer> menuIdList, String orderByStr) {
        if (CollectionUtils.isEmpty(menuIdList)) {
            return new ArrayList<>();
        }
        return tabSysMenuMapper.selectByIdsAndOrderBy(menuIdList, orderByStr);
    }

    @Override
    public List<AuthItemListDTO> listAuthItemByMenuId(Integer menuId) {
        return tabSysMenuMapper.listAuthItemByMenuId(menuId);
    }

    @Override
    public boolean isHasChild(Integer menuId) {
        int count = tabSysMenuMapper.countChildByParentId(menuId);
        if (count > 0) {
            return true;
        }
        return false;
    }

    @Override
    public List<TabSysMenu> listMenuTree(String project, String versionCode, List<Integer> authTypeList) {
        return tabSysMenuMapper.listMenuTree(project, versionCode, authTypeList);
    }

    @Override
    public List<TabSysMenu> selectByProjectList(List<String> projectList) {
        return tabSysMenuMapper.selectByProjectList(projectList);
    }

    @Override
    public List<TabSysMenu> selectAppMenuByParam(List<String> projectList, List<String> versionList, List<Integer> authTypeList) {
        return tabSysMenuMapper.selectAppMenuByParam(projectList, versionList, authTypeList);
    }

    @Override
    public boolean isRepeatMenuCode(String code, Integer menuId) {
        int count = tabSysMenuMapper.countByMenuCode(code, menuId);
        if (count > 0) {
            return true;
        }
        return false;
    }

    @Override
    public TabSysMenu getMenuByMenuCode(String menuCode) {
        return tabSysMenuMapper.getMenuByMenuCode(menuCode);
    }

    @Override
    public void updateParentCodeByParentId(Integer parentId, String parentCode) {
        tabSysMenuMapper.updateParentCodeByParentId(parentId, parentCode);
    }

    @Override
    public List<TabSysMenu> listAdminOnlyOperationItem() {
        return tabSysMenuMapper.listAdminOnlyOperationItem();
    }

    @Override
    public void deleteByMenuIdList(List<Integer> menuIdList) {
        if (CollectionUtils.isNotEmpty(menuIdList)) {
            tabSysMenuMapper.deleteByMenuIdList(menuIdList);
        }
    }

    @Override
    public List<TabSysMenu> listByParentMenuIdList(List<Integer> parentMenuIdList) {
        if (CollectionUtils.isEmpty(parentMenuIdList)) {
            return new ArrayList<>();
        }
        return tabSysMenuMapper.listByParentMenuIdList(parentMenuIdList);
    }

    @Override
    public void deleteNotInApp(List<String> projectList) {
        //目前删除platform_type 为XXX的数据，做到同步应用市场应用数据
        List<Integer> platformTypeList = new ArrayList<>(2);
        platformTypeList.add(MenuPlatformTypeEnum.GIC.getCode());
        platformTypeList.add(MenuPlatformTypeEnum.HAOBAN_APP.getCode());
        platformTypeList.add(MenuPlatformTypeEnum.HAOBAN_APPLET_APP.getCode());
        tabSysMenuMapper.deleteNotInApp(projectList, platformTypeList);
    }

    @Override
    public List<TabSysMenu> listMenuForGicContainChild(List<Integer> parentIdList) {
        if (CollectionUtils.isEmpty(parentIdList)) {
            return new ArrayList<>();
        }
        List<TabSysMenu> result = new ArrayList<>();
        List<TabSysMenu> list = tabSysMenuMapper.listByParentMenuIdList(parentIdList);
        while (CollectionUtils.isNotEmpty(list)) {
            result.addAll(list);
            List<Integer> menuIdList = list
                    .stream()
                    .filter(e -> e != null)
                    .map(e -> e.getMenuId())
                    .collect(Collectors.toList());
            list.clear();
            list = tabSysMenuMapper.listByParentMenuIdList(menuIdList);
        }
        return result;
    }

    @Override
    public void updateMenuTmp(List<Integer> menuIdList, String project, Integer parentId, String parentCode, String menuVersion, String menuVersionName) {
        tabSysMenuMapper.updateMenuTmp(menuIdList, project, parentId, parentCode, menuVersion, menuVersionName);
    }

    @Override
    public List<TabSysMenu> listMenuTemp(String project) {
        return tabSysMenuMapper.listMenuTemp(project);
    }

    @Override
    public List<TabSysMenu> listMenuTempForGic(String menuCode) {
        List<TabSysMenu> result = new ArrayList<>();
        List<TabSysMenu> firstLevelData = tabSysMenuMapper.listMenuTempByMenuCode(menuCode);
        if (CollectionUtils.isNotEmpty(firstLevelData)) {
            result.addAll(firstLevelData);
            List<TabSysMenu> temp = tabSysMenuMapper.listMenuTempByMenuIdList(firstLevelData
                    .stream()
                    .filter(e -> e != null)
                    .map(e -> e.getMenuId())
                    .collect(Collectors.toList()));
            while (CollectionUtils.isNotEmpty(temp)) {
                result.addAll(temp);
                List<Integer> parentIdList = temp
                        .stream()
                        .filter(e -> e != null)
                        .map(e -> e.getMenuId())
                        .collect(Collectors.toList());
                temp.clear();
                temp = tabSysMenuMapper.listMenuTempByMenuIdList(parentIdList);
            }
        }
        return result;
    }

    @Override
    public List<TabSysMenu> getAuthItemByDevMenuId(Integer menuId) {
        return tabSysMenuMapper.getAuthItemByDevMenuId(menuId);
    }

    private void updateSort(Integer sort, Integer id) {
        TabSysMenu record = new TabSysMenu();
        record.setMenuId(id);
        record.setSort(sort);
        tabSysMenuMapper.updateByPrimaryKeySelective(record);
    }
}
