package com.gic.auth.service.outer.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.application.center.api.dto.ApplicationResourceDTO;
import com.gic.application.center.api.service.AuthorizeService;
import com.gic.auth.constant.OrderOrAppEnum;
import com.gic.auth.dto.OrderResourceContentDTO;
import com.gic.auth.dto.ResourceDTO;
import com.gic.auth.dto.open.*;
import com.gic.auth.entity.TabSysOrderAppResource;
import com.gic.auth.service.OrderAppResourceService;
import com.gic.auth.service.ResourceApiService;
import com.gic.auth.service.ResourceOpenApiService;
import com.gic.commons.util.EntityUtil;
import com.gic.gateway.sdk.anno.GatewayExporter;
import com.gic.gateway.sdk.anno.GatewayParams;
import com.gic.goods.api.dto.selector.GoodsRightsSelectorWithRightsDTO;
import com.gic.goods.api.service.GoodsRightsSelectorApiService;
import com.gic.member.config.api.dto.AppletsConfigSimpleOutputDTO;
import com.gic.member.config.api.dto.ChannelAreaInfoDTO;
import com.gic.member.config.api.dto.OwnerMemberCardDTO;
import com.gic.member.config.api.dto.ServiceConfigSimpleOutputDTO;
import com.gic.member.config.api.service.AppletsConfigApiService;
import com.gic.member.config.api.service.AuthorizationEnterConfApiService;
import com.gic.member.config.api.service.MemberCardApiService;
import com.gic.member.config.api.service.ServiceConfigApiService;
import com.gic.member.filter.api.service.CuIndexOperaApiService;
import com.gic.store.dto.StoreDTO;
import com.gic.store.service.StoreWidgetApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 
 * @ClassName: ResourceOpenApiServiceImpl
 * @Description:  
 * @author zhiwj
 * @date 2020-09-10 9:38
 */
@Service("resourceOpenApiService")
@GatewayExporter
public class ResourceOpenApiServiceImpl implements ResourceOpenApiService{
    private static final Logger LOGGER = LogManager.getLogger(ResourceOpenApiServiceImpl.class);

    @Autowired
    private AuthorizeService authorizeService;
    @Autowired
    private ResourceApiService resourceApiService;
    @Autowired
    private StoreWidgetApiService storeWidgetApiService;
    @Autowired
    private CuIndexOperaApiService cuIndexOperaApiService;
    @Autowired
    private OrderAppResourceService orderAppResourceService;
    @Autowired
    private MemberCardApiService memberCardApiService;
    @Autowired
    private AppletsConfigApiService appletsConfigApiService;
    @Autowired
    private ServiceConfigApiService serviceConfigApiService;
    @Autowired
    private AuthorizationEnterConfApiService authorizationEnterConfApiService;
    @Autowired
    private GoodsRightsSelectorApiService goodsRightsSelectorApiService;

    @Override
    @GatewayParams(query = {"appId", "enterpriseId"})
    public ServiceResponse<ResourceOpenDTO> getResource(String appId, Integer enterpriseId) {
        ResourceDTO resourceDTO = getResourceDTO(appId, enterpriseId);
        ResourceOpenDTO result = new ResourceOpenDTO();
        //门店资源
        result.setStore(getStoreResource(enterpriseId, resourceDTO));
        //用户资源
        result.setUserResource(getUserResource(enterpriseId, resourceDTO));
        //渠道资源
        result.setChannelResource(getChannelResource(enterpriseId, resourceDTO));
        //订单资源
        result.setOrderResource(getOrderResource(enterpriseId, resourceDTO));
        //商品资源
        result.setGoodsResource(getGoodsResource(enterpriseId, resourceDTO));
        return ServiceResponse.success(result);
    }

    private GoodsResourceDTO getGoodsResource(Integer enterpriseId, ResourceDTO resourceDTO) {
        if (resourceDTO != null) {
            Long goodsResource = resourceDTO.getGoodsResourceId();
            GoodsResourceDTO dto = new GoodsResourceDTO();
            dto.setGoodsResource(goodsResource);
            if (goodsResource != null) {
                ServiceResponse<GoodsRightsSelectorWithRightsDTO> result = goodsRightsSelectorApiService
                        .getGoodsRightsSelectorWithRights(goodsResource, enterpriseId);
                LOGGER.info("对外商品资源查询接口:{}", JSON.toJSONString(result));
                if (result.isSuccess()) {
                    GoodsRightsSelectorWithRightsDTO goodsRightsSelectorWithRightsDTO = result.getResult();
                    GoodsRightsAuthDTO goodsRightsAuth = EntityUtil.changeEntityNew(GoodsRightsAuthDTO.class, goodsRightsSelectorWithRightsDTO);
                    dto.setGoodsRightsAuth(goodsRightsAuth);
                    return dto;
                }
            }
        }
        return null;
    }

    private OrderResourceDTO getOrderResource(Integer enterpriseId, ResourceDTO resourceDTO) {
        if (resourceDTO != null) {
            Integer orderResourceId = resourceDTO.getOrderResource();
            OrderResourceDTO dto = new OrderResourceDTO();
            dto.setOrderResource(orderResourceId);
            if (orderResourceId != null) {
                TabSysOrderAppResource sysOrderAppResource = orderAppResourceService.getById(orderResourceId);
                List<OrderResourceContentDTO> contentDTOList = JSON.parseArray(sysOrderAppResource.getContent(), OrderResourceContentDTO.class);
                if (CollectionUtils.isNotEmpty(contentDTOList)) {
                    List<OrderStoreDTO> orderStoreList = new ArrayList<>();
                    for (OrderResourceContentDTO orderResourceContentDTO : contentDTOList) {
                        if (OrderOrAppEnum.ORDER.getCode().equals(orderResourceContentDTO.getChannel())) {
                            //查询门店资源组所有的门店权限
                            ServiceResponse<Page<StoreDTO>> storeResult = storeWidgetApiService
                                    .listStoreByStoreWidgetId(enterpriseId, orderResourceContentDTO.getStoreWidgetId(), 1, 10000);
                            if (storeResult.isSuccess()) {
                                Page<StoreDTO> page = storeResult.getResult();
                                if (page == null) {
                                    return null;
                                }
                                List<StoreDTO> list = page.getResult();
                                if (CollectionUtils.isNotEmpty(list)) {
                                    for (StoreDTO storeDTO : list) {
                                        orderStoreList.add(new OrderStoreDTO(storeDTO.getStoreId(), storeDTO.getStoreName(), 1));
                                    }
                                }
                            }
                        }
                    }
                    dto.setOrderStoreAuth(new OrderStoreAuthDTO(orderStoreList));
                    return dto;
                }
            }
        }
        return null;
    }

    private ChannelResourceDTO getChannelResource(Integer enterpriseId, ResourceDTO resourceDTO) {
        if (resourceDTO != null) {
            Integer channelResourceId = resourceDTO.getChannelResource();
            if (channelResourceId != null) {
                TabSysOrderAppResource channelResource = orderAppResourceService.getById(channelResourceId);
                if (channelResource != null) {
                    ChannelResourceDTO dto = new ChannelResourceDTO();
                    //渠道资源ID
                    dto.setChannelResource(channelResourceId);

                    String content = channelResource.getContent();
                    if (StringUtils.isNotBlank(content)) {
                        JSONObject json = (JSONObject) JSONObject.parse(content);
                        List<Long> memberCardList = (List<Long>) json.get("cardConfigIdList");
                        List<Long> appList = (List<Long>) json.get("appletConfigIdList");
                        List<Long> serviceList = (List<Long>) json.get("serviceConfigIdList");

                        Map<String, String> appletMap = getAppletMap(enterpriseId);
                        Map<String, String> memberCardMap = getMemberCardMap(enterpriseId);
                        Map<String, String> serviceMap = getServiceMap(enterpriseId);

                        List<ChannelDTO> channelList = new ArrayList<>();
                        //1：会员卡
                        if (CollectionUtils.isNotEmpty(memberCardList)) {
                            for (Long str : memberCardList) {
                                channelList.add(new ChannelDTO(str, memberCardMap.get(str.toString()), 1));
                            }
                        }
                        //2：服务号
                        if (CollectionUtils.isNotEmpty(appList)) {
                            for (Long str : appList) {
                                channelList.add(new ChannelDTO(str, serviceMap.get(str.toString()), 2));
                            }
                        }
                        //3：小程序
                        if (CollectionUtils.isNotEmpty(serviceList)) {
                            for (Long str : serviceList) {
                                channelList.add(new ChannelDTO(str, appletMap.get(str.toString()), 3));
                            }
                        }
                        dto.setChannelAuth(new ChannelAuthDTO(channelList));
                        return dto;
                    }
                } else {
                    LOGGER.warn("对外渠道资源查询接口为空");
                }
            }
        }
        return null;
    }

    private Map<String, String> getAppletMap(Integer enterpriseId) {
        Map<String, String> resultMap = new HashMap<>(16);
        ServiceResponse<List<AppletsConfigSimpleOutputDTO>> result = appletsConfigApiService.listValidAppletConfigAppId(enterpriseId);
        if (result.isSuccess()) {
            List<AppletsConfigSimpleOutputDTO> list = result.getResult();
            if (CollectionUtils.isNotEmpty(list)) {
                for (AppletsConfigSimpleOutputDTO dto : list) {
                    resultMap.put(dto.getAppletConfigId().toString(), dto.getAppletConfigName());
                }
            }
        }
        return resultMap;
    }

    private Map<String, String> getMemberCardMap(Integer enterpriseId) {
        Map<String, String> resultMap = new HashMap<>(16);
        ServiceResponse<List<OwnerMemberCardDTO>> result = memberCardApiService.getAll(enterpriseId);
        if (result.isSuccess()) {
            List<OwnerMemberCardDTO> list = result.getResult();
            if (CollectionUtils.isNotEmpty(list)) {
                for (OwnerMemberCardDTO dto : list) {
                    resultMap.put(dto.getCardConfigId().toString(), dto.getCardName());
                }
            }
        }
        return resultMap;
    }

    private Map<String, String> getServiceMap(Integer enterpriseId) {
        Map<String, String> resultMap = new HashMap<>(16);
        ServiceResponse<List<ServiceConfigSimpleOutputDTO>> result = serviceConfigApiService.listValidServiceConfigAppId(enterpriseId);
        if (result.isSuccess()) {
            List<ServiceConfigSimpleOutputDTO> list = result.getResult();
            if (CollectionUtils.isNotEmpty(list)) {
                for (ServiceConfigSimpleOutputDTO dto : list) {
                    resultMap.put(dto.getServiceConfigId().toString(), dto.getServiceConfigName());
                }
            }
        }
        return resultMap;
    }

    private UserResourceDTO getUserResource(Integer enterpriseId, ResourceDTO resourceDTO) {
        if (resourceDTO != null) {
            Long userResourceId = resourceDTO.getUserResource();
            if (userResourceId != null) {
                ServiceResponse<List<ChannelAreaInfoDTO>> userResult = authorizationEnterConfApiService.getAreaInfoByScreenWidget(userResourceId, enterpriseId);
                LOGGER.info("对外用户资源查询接口:{}", JSON.toJSONString(userResult));
                if (userResult.isSuccess()) {
                    UserResourceDTO dto = new UserResourceDTO();
                    dto.setUserResource(userResourceId);
                    List<ChannelAreaInfoDTO> userResource = userResult.getResult();
                    if (CollectionUtils.isNotEmpty(userResource)) {
                        List<UserAreaDTO> userAreaList = new ArrayList<>();
                        for (ChannelAreaInfoDTO channelAreaInfoDTO : userResource) {
                            userAreaList.add(new UserAreaDTO(channelAreaInfoDTO.getAreaId(), channelAreaInfoDTO.getName(), channelAreaInfoDTO.getCuType()));
                        }
                        dto.setUserArea(new UserAreaAuthDTO(userAreaList));
                        return dto;
                    }
                }
            }
        }
        return null;
    }


    private StoreResourceDTO getStoreResource(Integer enterpriseId, ResourceDTO resourceDTO) {
        List<StoreOpenDTO> storeList = new ArrayList<>();
        if (resourceDTO != null) {
            Long storeResourceId = resourceDTO.getStoreResource();
            //查询门店资源组所有的门店权限
            ServiceResponse<Page<StoreDTO>> storeResult = storeWidgetApiService
                    .listStoreByStoreWidgetId(enterpriseId, storeResourceId.intValue(), 1, 10000);
            if (storeResult.isSuccess()) {
                Page<StoreDTO> page = storeResult.getResult();
                if (page == null) {
                    return null;
                }
                List<StoreDTO> list = page.getResult();
                Map<String, String> regionMap = new HashMap<>(16);
                if (CollectionUtils.isNotEmpty(list)) {
                    for (StoreDTO storeDTO : list) {
                        regionMap.put(storeDTO.getRegionId().toString(), storeDTO.getRegionName());
                    }
                    for(Map.Entry<String, String> entry : regionMap.entrySet()) {
                        storeList.add(new StoreOpenDTO(Integer.parseInt(entry.getKey()), entry.getValue()));
                    }
                    StoreAuthDTO storeAuth = new StoreAuthDTO(storeList);
                    return new StoreResourceDTO(storeResourceId, storeAuth);
                }
            }
        }
        return null;
    }

    private ResourceDTO getResourceDTO(String appId, Integer enterpriseId) {
        ServiceResponse<ApplicationResourceDTO> res = authorizeService
                .findResourceByEnterpriseIdAndAppId(Long.valueOf(enterpriseId), appId);
        LOGGER.info("对外门店资源查询接口:{}", JSON.toJSONString(res));
        if (res.isSuccess()) {
            ApplicationResourceDTO resourceDTO = res.getResult();
            if (resourceDTO != null) {
                Integer resourceId = resourceDTO.getResourceId();
                if (resourceId == null) {
                    return null;
                }
                ServiceResponse<ResourceDTO> resourceResult = resourceApiService.getResource(resourceId);
                LOGGER.info("对外API门店资源组数据:{}", JSON.toJSONString(resourceResult));
                if (resourceResult.isSuccess()) {
                    return resourceResult.getResult();
                }
            }
        }
        return null;
    }
}
