package com.gic.auth.web.controller;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.ResourceDTO;
import com.gic.auth.service.ResourceApiService;
import com.gic.auth.web.qo.OrderResourceQO;
import com.gic.auth.web.qo.PageQO;
import com.gic.auth.web.qo.ResourceQO;
import com.gic.auth.web.vo.*;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.log.LogUtils;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.goods.api.service.GoodsRightsSelectorApiService;
import com.gic.mall.share.api.dto.shop.ShopDTO;
import com.gic.mall.share.api.enums.ShopTypeEnum;
import com.gic.mall.share.api.service.ShopApiService;
import com.gic.marketing.api.dto.sms.EnterpriseSmsSignDTO;
import com.gic.marketing.api.service.sms.SmsApiService;
import com.gic.member.config.api.dto.AppletsConfigSimpleOutputDTO;
import com.gic.member.config.api.dto.OwnerMemberCardDTO;
import com.gic.member.config.api.dto.ServiceConfigSimpleOutputDTO;
import com.gic.member.config.api.service.AppletsConfigApiService;
import com.gic.member.config.api.service.MemberCardApiService;
import com.gic.member.config.api.service.ServiceConfigApiService;
import com.gic.open.api.dto.EnterpriseServeInfoDTO;
import com.gic.open.api.service.AppTokenApiService;
import com.gic.open.api.service.EnterpriseLicenseApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019/8/29 9:09
 */
@RestController
public class ResourceController {

    @Autowired
    private ResourceApiService resourceApiService;
    @Autowired
    private GoodsRightsSelectorApiService goodsRightsSelectorApiService;
    @Autowired
    private SmsApiService smsApiService;
    @Autowired
    private AppTokenApiService appTokenApiService;
    @Autowired
    private MemberCardApiService memberCardApiService;
    @Autowired
    private AppletsConfigApiService appletsConfigApiService;
    @Autowired
    private ServiceConfigApiService serviceConfigApiService;
    @Autowired
    private ShopApiService shopApiService;
    @Autowired
    private EnterpriseLicenseApiService enterpriseLicenseApiService;


    @RequestMapping("/list-sms-sign")
    public RestResponse listSmsSign(String search) {
        ServiceResponse<List<EnterpriseSmsSignDTO>> response = smsApiService
                .listSignByEnterprise(UserDetailUtils.getUserDetail().getEnterpriseId(), search);
        if (response.isSuccess()) {
            return RestResponse.success(EntityUtil.changeEntityListByJSON(EnterpriseSmsSignVO.class, response.getResult()));
        } else {
            return RestResponse.failure(response.getCode(), response.getMessage());
        }
    }

    @RequestMapping("/save-resource")
    public RestResponse saveResource(@RequestBody ResourceQO resourceQO) {
        ResourceDTO resourceDTO = EntityUtil.changeEntityByJSON(ResourceDTO.class, resourceQO);
        resourceDTO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());

        // 订单资源
        if (CollectionUtils.isNotEmpty(resourceQO.getOrderResourceJson())) {
            List<OrderResourceQO> orderResource = resourceQO.getOrderResourceJson();
            resourceDTO.setOrderResourceJson(JSON.toJSONString(orderResource));
        }
        // 短信签名资源
        if (resourceQO.getSmsResourceJson() != null) {
            resourceDTO.setSmsSignResourceJson(JSON.toJSONString(resourceQO.getSmsResourceJson()));
        }
        if (resourceQO.getChannelResourceVO() != null) {
            resourceDTO.setChannelResourceJson(JSON.toJSONString(resourceQO.getChannelResourceVO()));
        }

        ServiceResponse<Integer> response = resourceApiService.saveOrUpdateResource(resourceDTO);
        if (response.isSuccess()) {
            LogUtils.createLog((resourceQO.getResourceId() == null ? "新增" : "修改") + "资源组",
                    resourceQO.getResourceName());
            return RestResponse.success(response.getResult());
        }
        return EnterpriseRestResponse.failure(response);
    }

    @RequestMapping("/del-resource")
    public RestResponse delResource(Integer resourceId) {
        ServiceResponse<ResourceDTO> resource = resourceApiService.getResource(resourceId);
        ServiceResponse<Integer> serviceResponse = resourceApiService.delResource(resourceId);
        if (resource.isSuccess()) {
            LogUtils.createLog("删除资源组", resource.getResult().getResourceName());
        }
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/list-resource")
    public RestResponse listResource(ResourceQO resourceQO, PageQO pageQO) {
        ResourceDTO resourceDTO = EntityUtil.changeEntityByOrika(ResourceDTO.class, resourceQO);
        resourceDTO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        ServiceResponse<Page<ResourceDTO>> serviceResponse = resourceApiService.listResource(resourceDTO,
                pageQO.getCurrentPage(), pageQO.getPageSize());
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/get-resource")
    public RestResponse getResource(Integer resourceId) {
        ServiceResponse<ResourceDTO> response = resourceApiService.getResource(resourceId);
        if (response.isSuccess()) {
            ResourceDTO dto = response.getResult();
            ResourceVO vo = EntityUtil.changeEntity(ResourceVO.class, dto);
            if (StringUtils.isNotBlank(dto.getOrderResourceJson())) {
                List<OrderResourceVO> orderResourceList = JSON.parseArray(dto.getOrderResourceJson(),
                        OrderResourceVO.class);
                vo.setOrderResourceJson(orderResourceList);
            }
            if (StringUtils.isNotBlank(dto.getSmsSignResourceJson())) {
                SmsSignResourceVO smsSignResourceDTO = JSON.parseObject(dto.getSmsSignResourceJson(),
                        SmsSignResourceVO.class);
                vo.setSmsResourceJson(smsSignResourceDTO);
            }
            if (StringUtils.isNotBlank(dto.getChannelResourceJson())) {
                ChannelResourceVO channelResourceDTO = JSON.parseObject(dto.getChannelResourceJson(),
                        ChannelResourceVO.class);
                vo.setChannelResourceJson(channelResourceDTO);
            }
            return RestResponse.success(vo);
        }
        return EnterpriseRestResponse.failure(response);
    }

    @RequestMapping("/channel-resource")
    public RestResponse channelResource() {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        List<OwnerMemberCardDTO> memberCardList = memberCardApiService.getAll(enterpriseId).getResult();
        List<AppletsConfigSimpleOutputDTO> appletList = appletsConfigApiService.listValidAppletConfigAppId(enterpriseId)
                .getResult();
        List<ServiceConfigSimpleOutputDTO> serviceList = serviceConfigApiService
                .listValidServiceConfigAppId(enterpriseId).getResult();
        Map<String, Object> map = new HashMap<>(4);
        map.put("memberCardList", EntityUtil.changeEntityListByJSON(OwnerMemberCardVO.class, memberCardList));
        map.put("appletList", EntityUtil.changeEntityListByJSON(AppletsConfigSimpleOutputVO.class, appletList));
        map.put("serviceList", EntityUtil.changeEntityListByJSON(ServiceConfigSimpleOutputVO.class, serviceList));
        return RestResponse.success(map);
    }

    @RequestMapping("/gic-shop-resource")
    public RestResponse gicShopResource() {
        ServiceResponse<List<ShopDTO>> serviceResponse = shopApiService.getAllShopByEnterpriseIdAndName(UserDetailUtils.getUserDetail().getEnterpriseId(), null, ShopTypeEnum.MALL_SHOP.getCode());
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/single-version")
    public RestResponse isSingle() {
        ServiceResponse<EnterpriseServeInfoDTO> serveInfo = enterpriseLicenseApiService.getServeInfo(UserDetailUtils.getUserDetail().getEnterpriseId(), "gic");
        return RestResponse.success("basic".equals(serveInfo.getResult().getVersionCode()));
    }
 }
